import React from 'react'
import ReactDOM from 'react-dom/client'

// Get useSyncExternalStore directly to ensure it's available
import * as useSyncExternalStoreExports from 'use-sync-external-store/shim/with-selector'

// Assign to global scope IMMEDIATELY after import
window.React = React;
window.ReactDOM = ReactDOM;

// Ensure useSyncExternalStore is available - critical for Zustand
if (!window.React.useSyncExternalStore && useSyncExternalStoreExports) {
  window.React.useSyncExternalStore = useSyncExternalStoreExports.useSyncExternalStoreWithSelector;
  console.log('%c[App Initialization] Installed useSyncExternalStore from shim', 'color: #0047AB');
}

import App from './App.tsx'
import './index.css'
import { initializeAnalytics, initializeMarketing, applyCookiePreferences } from './lib/cookieService';
import { useCookieStore, isCookieAllowed } from './store/cookies';

// Debug info
console.log('%c[App Initialization] React object assigned to window:', 'color: #0047AB', window.React);
console.log('%c[App Initialization] useSyncExternalStore available:', 'color: #0047AB', !!window.React.useSyncExternalStore);

// Debug the cookie store state during initialization
console.log('%c[App Initialization] Starting app', 'background: #0047AB; color: white; font-weight: bold');

// Check if localStorage is working
try {
  localStorage.setItem('app-init-test', 'test');
  const testResult = localStorage.getItem('app-init-test');
  console.log('%c[App Initialization] localStorage test:', 'color: #0047AB', testResult === 'test' ? 'PASSED' : 'FAILED');
  localStorage.removeItem('app-init-test');
} catch (e) {
  console.error('%c[App Initialization] localStorage ERROR:', 'color: red', e);
}

// Log cookie store initialization
const cookieState = useCookieStore.getState();
console.log('%c[App Initialization] Initial cookie state:', 'color: #0047AB', {
  consentGiven: cookieState.consentGiven,
  hasShownBanner: cookieState.hasShownBanner,
  preferences: cookieState.preferences
});

// Debug persisted storage content
console.log('%c[App Initialization] LocalStorage cookie data:', 'color: #0047AB', 
  localStorage.getItem('funcart-cookie-preferences')
);

// Only initialize services if user has given consent and selected the respective preferences
if (cookieState.consentGiven === true) {
  console.log('%c[App Initialization] User has given consent, initializing services', 'color: #00A36C');
  
  // Apply cookie preferences from stored settings
  applyCookiePreferences();
  
  if (isCookieAllowed('analytics')) {
    console.log('%c[App Initialization] Initializing analytics', 'color: #00A36C');
    initializeAnalytics();
  }
  
  if (isCookieAllowed('marketing')) {
    console.log('%c[App Initialization] Initializing marketing', 'color: #00A36C');
    initializeMarketing();
  }
} else {
  console.log('%c[App Initialization] User has not given consent or undecided, skipping service initialization', 'color: #D3212D');
}

// Initialize service worker if available
if ('serviceWorker' in navigator) {
  window.addEventListener('load', () => {
    navigator.serviceWorker
      .register('/serviceWorker.js')
      .then(registration => {
        console.log('ServiceWorker registration successful');
      })
      .catch(err => {
        console.log('ServiceWorker registration failed: ', err);
      });
  });
}

// Flag to track if React has been rendered
let reactRendered = false;

// Ensure the DOM is fully loaded before rendering the React app
const renderApp = () => {
  const root = document.getElementById('root');
  if (root) {
    // Verify React is ready before rendering
    if (!window.React?.useState) {
      console.error('%c[App Initialization] React hooks not available yet before render. Retrying...', 'color: red');
      setTimeout(renderApp, 50);
      return;
    }
    
    ReactDOM.createRoot(root).render(
      <React.StrictMode>
        <App />
      </React.StrictMode>
    );
    reactRendered = true;
    console.log('%c[App Initialization] React app rendered.', 'color: #00A36C');
    
    // Load Google Maps API after React is initialized
    setTimeout(() => {
      if (typeof window.loadGoogleMapsApi === 'function') {
        console.log('%c[App Initialization] Attempting to load Google Maps API...', 'color: #00A36C');
        window.loadGoogleMapsApi();
      } else {
        console.error('%c[App Initialization] window.loadGoogleMapsApi not found', 'color: red');
      }
    }, 500); // Delay to allow React initialization
  } else {
    console.error('Root element not found, retrying in 10ms');
    setTimeout(renderApp, 10);
  }
};

if (document.readyState === 'loading') {
  document.addEventListener('DOMContentLoaded', renderApp);
} else {
  renderApp();
}

// Declare global interfaces
declare global {
  interface Window {
    React: typeof React;
    ReactDOM: typeof ReactDOM;
    google?: any;
    googleMapsLoaded?: boolean;
    initMap: () => void;
    loadGoogleMapsApi: () => void;
    GOOGLE_MAPS_API_KEY: string;
    // Removed require/module
  }
}
